#!/usr/bin/env python3
# SPDX-License-Identifier: LGPL-2.1-or-later

from argparse import ArgumentParser
import glob
import json
import os
import re
import subprocess
import sys

import requests

BASE_URL = "https://www.freedesktop.org/software/systemd/man/"
JQUERY_URL = "https://code.jquery.com/jquery-3.7.1.min.js"
SCRIPT_TAG = '<script src="{}"></script>'

NAV_JS = """
$(document).ready(function() {
    $.getJSON("../index.json", function(data) {
        data.sort().reverse();

        var [filename, dirname] = window.location.pathname.split("/").reverse();

        var items = [];
        $.each( data, function(_, version) {
            if (version == data[1]) {
                latest = " (latest stable)";
            } else {
                latest = "";
            }
            if (version == dirname) {
                items.push( "<option selected value='" + version + "'>" + "elogind " + version + "</option>");
                items.push( "<option selected value='" + version + "'>" + "systemd " + version + latest + "</option>");
            } else if (dirname == "latest" && version == data[1]) {
                items.push( "<option selected value='" + version + "'>" + "elogind " + version + "</option>");
                items.push( "<option selected value='" + version + "'>" + "systemd " + version + latest + "</option>");
            } else {
                items.push( "<option value='" + version + "'>" + "elogind " + version + "</option>");
                items.push( "<option value='" + version + "'>" + "systemd " + version + latest + "</option>");
            }
        });

        $("span:first").html($( "<select/>", {
            id: "version-selector",
            html: items.join( "" )
        }));

        $("#version-selector").on("change", function() {
            window.location.assign("../" + $(this).val() + "/" + filename);
        });
    });
});
"""


def process_file(filename):
    with open(filename) as f:
        contents = f.read()

    if SCRIPT_TAG.format("../nav.js") in contents:
        return

    body_tag = re.search("<body[^>]*>", contents)
    new_contents = (
        contents[: body_tag.end()]
        + SCRIPT_TAG.format(JQUERY_URL)
        + SCRIPT_TAG.format("../nav.js")
        + contents[body_tag.end() :]
    )

    with open(filename, "w") as f:
        f.write(new_contents)


def update_index_file(version, index_filename):
    response = requests.get(BASE_URL + "index.json")
    if response.status_code == 404:
        index = []
    elif response.ok:
        index = response.json()
    else:
        sys.exit(f"Error getting index: {response.status_code} {response.reason}")

    if version not in index:
        index.insert(0, version)

    with open(index_filename, "w") as f:
        json.dump(index, f)


def get_latest_version():
    tags = subprocess.check_output(["git", "tag", "-l", "v*"], text=True).split()
    versions = []
    for tag in tags:
        m = re.match(r"v?(\d+).*", tag)
        if m:
            versions.append(int(m.group(1)))
    return max(versions)


def main(version, directory, www_target):
    index_filename = os.path.join(directory, "index.json")
    nav_filename = os.path.join(directory, "nav.js")
    # The upload directory does not contain point release suffixes
    version = re.sub(r"\..+$", "", version)

    current_branch = subprocess.check_output(["git", "branch", "--show-current"], text=True).strip()

    if current_branch != 'main' and not current_branch.endswith("-stable"):
        sys.exit("doc-sync should only be run from main or a stable branch")

    for filename in glob.glob(os.path.join(directory, "*.html")):
        process_file(filename)

    if current_branch == "main":
        version = "devel"
        dirs = ["devel"]
    elif int(version) == get_latest_version():
        dirs = [version, "latest"]
    else:
        dirs = [version]

    with open(nav_filename, "w") as f:
        f.write(NAV_JS)

    update_index_file(version, index_filename)

    for d in dirs:
        subprocess.check_call(
            [
                "rsync",
                "-rlv",
                "--delete-excluded",
                "--include=*.html",
                "--exclude=*",
                "--omit-dir-times",
                directory + "/",  # copy contents of directory
                os.path.join(www_target, "man", d),
            ]
        )

    subprocess.check_call(
        [
            "rsync",
            "-v",
            os.path.join(directory, "index.json"),
            os.path.join(directory, "nav.js"),
            os.path.join(www_target, "man"),
        ]
    )


if __name__ == "__main__":
    parser = ArgumentParser()
    parser.add_argument("--version", required=True)
    parser.add_argument("directory")
    parser.add_argument("www_target")

    args = parser.parse_args()
    main(args.version, args.directory, args.www_target)



#!/usr/bin/env python3
# SPDX-License-Identifier: LGPL-2.1-or-later
# pylint: disable=unbalanced-tuple-unpacking,consider-using-f-string,consider-using-with

"""
Generate %-from-name.gperf from %-list.txt
"""

import sys

if __name__ == '__main__':
    if len(sys.argv) != 4:
        sys.exit(f'Usage: {sys.argv[0]} name prefix file')

    name, prefix, file = sys.argv[1:]

    print("""\
%{
#if __GNUC__ >= 7
_Pragma("GCC diagnostic ignored \\"-Wimplicit-fallthrough\\"")
#endif
%}""")
    print(f"""\
struct {name}_name {{ const char* name; int id; }};
%null-strings
%%""")

    for line in open(file):
        print("{0}, {1}{0}".format(line.rstrip(), prefix))



#!/usr/bin/env python3
# SPDX-License-Identifier: LGPL-2.1-or-later

# pylint: disable=consider-using-with

import os
import pathlib
import re
import sys

PROJECT_ROOT = pathlib.Path(os.getenv('PROJECT_SOURCE_ROOT', '.'))

def check_file(filename):
    seen = set()
    good = True
    for n, line in enumerate(open(filename)):
        m = re.match(r'^\s*#\s*include\s*[<"](\S*)[>"]', line)
        if m:
            include = m.group(1)
            if include in seen:
                try:
                    filename = pathlib.Path(filename).resolve().relative_to(PROJECT_ROOT)
                except ValueError:
                    pass
                print(f'{filename}:{n}: {line.strip()}')
                good = False
            seen.add(include)
    return good

if __name__ == '__main__':
    all_good = all(check_file(name) for name in sys.argv[1:])
    sys.exit(0 if all_good else 1)



#!/usr/bin/env python3
# SPDX-License-Identifier: LGPL-2.1-or-later

import os
import re
import sys

def process_sym_file(file):
    for line in file:
        m = re.search(r'^ +([a-zA-Z0-9_]+);', line)
        if m:
            if m[1] == 'sd_bus_object_vtable_format':
                print('        {{"{0}", &{0}}},'.format(m[1]))
            else:
                print('        {{"{0}", {0}}},'.format(m[1]))

def process_source_file(file):
    for line in file:
        # Functions
        m = re.search(r'^_public_\s+(\S+\s+)+\**(\w+)\s*\(', line)
        if m:
            print('        {{ "{0}", {0} }},'.format(m[2]))
        # Variables
        m = re.search(r'^_public_\s+(\S+\s+)+\**(\w+)\s*=', line)
        if m:
            print('        {{ "{0}", &{0} }},'.format(m[2]))
        # Functions defined through a macro
        m = re.search(r'^DEFINE_PUBLIC_TRIVIAL_REF_FUNC\([^,]+,\s*(\w+)\s*\)', line)
        if m:
            print('        {{ "{0}_ref", {0}_ref }},'.format(m[1]))
        m = re.search(r'^DEFINE_PUBLIC_TRIVIAL_UNREF_FUNC\([^,]+,\s*(\w+)\s*,', line)
        if m:
            print('        {{ "{0}_unref", {0}_unref }},'.format(m[1]))
        m = re.search(r"^DEFINE_PUBLIC_TRIVIAL_REF_UNREF_FUNC\([^,]+,\s*(\w+)\s*,", line)
        if m:
            print('        {{ "{0}_ref", {0}_ref }},'.format(m[1]))
            print('        {{ "{0}_unref", {0}_unref }},'.format(m[1]))

print('''/* SPDX-License-Identifier: LGPL-2.1-or-later */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

''')

for header in sys.argv[3:]:
    print('#include "{}"'.format(header.split('/')[-1]))

print('''
/* We want to check deprecated symbols too, without complaining */
#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
''')

print('''
struct symbol {
        const char *name;
        const void *symbol;
};
static struct symbol symbols_from_sym[] = {''')

with open(sys.argv[1], "r") as f:
    process_sym_file(f)

print('''        {}
}, symbols_from_source[] = {''')

for dirpath, _, filenames in sorted(os.walk(sys.argv[2])):
    for filename in sorted(filenames):
        if not filename.endswith(".c") and not filename.endswith(".h"):
            continue
        with open(os.path.join(dirpath, filename), "r") as f:
            process_source_file(f)

print('''        {}
};

static int sort_callback(const void *a, const void *b) {
        const struct symbol *x = a, *y = b;
        return strcmp(x->name, y->name);
}

int main(void) {
        size_t i, j;

        qsort(symbols_from_sym, sizeof(symbols_from_sym)/sizeof(symbols_from_sym[0])-1, sizeof(symbols_from_sym[0]), sort_callback);
        qsort(symbols_from_source, sizeof(symbols_from_source)/sizeof(symbols_from_source[0])-1, sizeof(symbols_from_source[0]), sort_callback);

        puts("From symbol file:");
        for (i = 0; symbols_from_sym[i].name; i++)
                printf("%p: %s\\n", symbols_from_sym[i].symbol, symbols_from_sym[i].name);

        puts("\\nFrom source files:");
        for (j = 0; symbols_from_source[j].name; j++)
                printf("%p: %s\\n", symbols_from_source[j].symbol, symbols_from_source[j].name);

        puts("");
        printf("Found %zu symbols from symbol file.\\n", i);
        printf("Found %zu symbols from source files.\\n", j);

        for (i = 0; symbols_from_sym[i].name; i++) {
                struct symbol*n = bsearch(symbols_from_sym+i, symbols_from_source, sizeof(symbols_from_source)/sizeof(symbols_from_source[0])-1, sizeof(symbols_from_source[0]), sort_callback);
                if (!n)
                        printf("Found in symbol file, but not in sources: %s\\n", symbols_from_sym[i].name);
        }

        for (j = 0; symbols_from_source[j].name; j++) {
                struct symbol*n = bsearch(symbols_from_source+j, symbols_from_source, sizeof(symbols_from_sym)/sizeof(symbols_from_sym[0])-1, sizeof(symbols_from_sym[0]), sort_callback);
                if (!n)
                        printf("Found in sources, but not in symbol file: %s\\n", symbols_from_source[i].name);
        }

        return i == j ? EXIT_SUCCESS : EXIT_FAILURE;
}''')
